use std::str::Bytes;

pub struct U32s<'a> {
    bytes: Bytes<'a>,
}

impl<'a> Iterator for U32s<'a> {
    type Item = u32;

    fn next(&mut self) -> Option<Self::Item> {
        try_parse_u32(&mut self.bytes)
    }
}

pub trait ParseExt {
    fn parse_u32(&self) -> u32;

    fn u32s(&self) -> U32s<'_>;
}

impl ParseExt for &str {
    fn parse_u32(&self) -> u32 {
        match try_parse_u32(&mut self.bytes()) {
            Some(num) => num,
            None => panic!("unable to parse u32: {self}"),
        }
    }

    fn u32s(&self) -> U32s<'_> {
        U32s {
            bytes: self.bytes(),
        }
    }
}

fn try_parse_u32(bytes: &mut Bytes<'_>) -> Option<u32> {
    let mut n = loop {
        let byte = bytes.next()?;
        let digit = byte.wrapping_sub(b'0');

        if digit < 10 {
            break digit as u32;
        }
    };

    loop {
        let Some(byte) = bytes.next() else {
            break Some(n);
        };
        let digit = byte.wrapping_sub(b'0');

        if digit < 10 {
            n = 10 * n + digit as u32;
        } else {
            break Some(n);
        }
    }
}
