use crate::util::parse::ParseExt;

#[inline]
fn valid_triangle(v: &[u32; 3]) -> bool {
    v[0] + v[1] > v[2] && v[1] + v[2] > v[0] && v[0] + v[2] > v[1]
}

pub fn part1(input: &str) -> impl std::fmt::Display {
    input
        .lines()
        .map(|s| {
            s.u32s()
                // This is faster than `collect().try_into().expect("")`
                // presumably because of the allocation of the Vec
                // and the additional check for the Option for each line
                // incure a bit of overhead
                .fold(([0u32; 3], 0), |(mut a, i), v| {
                    a[i] = v;
                    (a, i + 1)
                })
                .0
        })
        .filter(valid_triangle)
        .count()
}

pub fn part2(input: &str) -> impl std::fmt::Display {
    fn get3<'a>(lines: &'a mut std::str::Lines<'_>) -> Option<(&'a str, &'a str, &'a str)> {
        Some((lines.next()?, lines.next()?, lines.next()?))
    }

    let mut lines = input.lines();
    let mut valid = 0;

    while let Some((l1, l2, l3)) = get3(&mut lines) {
        let l1: Vec<u32> = l1.u32s().collect();
        let l2: Vec<u32> = l2.u32s().collect();
        let l3: Vec<u32> = l3.u32s().collect();
        for i in 0..2 {
            if valid_triangle(&[l1[i], l2[i], l3[i]]) {
                valid += 1;
            }
        }
    }
    valid
}

#[test]
fn test_part1() {
    assert_eq!("0", part1("5 10 25").to_string())
}

#[test]
fn test_part2() {
    assert_eq!("2", part2("100 200 300\n101 201 301\n102 202 302").to_string());
}
